' //////////////////////////////////////////////////////////////////////
' Game Start
' AUTHOR: Frdrick Blais
' 30.8.09       Made assets and first tile map
' 31.8.09       Added gamepad for the balloon and lava animation
' 02.9.09       Added enemy fireball flying horizontally in the screen and lives
' 05.9.09       Vertical fireballs, monster dying animation, pause 
'
' //////////////////////////////////////////////////////////////////////

'///////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

CON

  _clkmode = xtal2 + pll8x          ' enable external clock and pll times 8
  _xinfreq = 10_000_000 + 0000      ' set frequency to 10 MHZ plus some error    
  _stack   = 64                     ' stack
  _memstart = $10                   ' memory starts $10 in!!! (this took 2 headaches to figure out)

  obj_n           = 24                                  ' Number of Objects
  obj_size        = 6                                   ' registers per object.
  obj_total_size = (obj_n*obj_size)                     ' Total Number of registers (LONGS)

  OBJ_OFFSET_X  = 0
  OBJ_OFFSET_Y  = 1
  OBJ_OFFSET_W  = 2
  OBJ_OFFSET_H  = 3
  OBJ_OFFSET_I  = 4
  OBJ_OFFSET_M  = 5
  KONST         = 2

  #0, h_cop_status, h_cop_control, h_cop_debug, h_cop_phase0, h_cop_monitor0, h_cop_monitor1, h_cop_config, h_cop_vram, h_cop_tile, h_cop_panx, h_cop_pany, h_cop_bgcolor, h_cop_obj

  FRAMERATE = 60                    ' 60FPS!!!

   ' NES bit encodings for NES gamepad 0
  NES0_RIGHT  = %00000000_00000001
  NES0_LEFT   = %00000000_00000010
  NES0_DOWN   = %00000000_00000100
  NES0_UP     = %00000000_00001000
  NES0_START  = %00000000_00010000
  NES0_SELECT = %00000000_00100000
  NES0_B      = %00000000_01000000
  NES0_A      = %00000000_10000000

  ' NES bit encodings for NES gamepad 1
  NES1_RIGHT  = %00000001_00000000
  NES1_LEFT   = %00000010_00000000
  NES1_DOWN   = %00000100_00000000
  NES1_UP     = %00001000_00000000
  NES1_START  = %00010000_00000000
  NES1_SELECT = %00100000_00000000
  NES1_B      = %01000000_00000000
  NES1_A      = %10000000_00000000

  SCREEN_HEIGHT = 220 
  SCREEN_WIDTH  = 240
  
'///////////////////////////////////////////////////////////////////////
' VARIABLES SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

' COP HEADER ------------------------------------------------------------------

VAR

long  cop_status
long  cop_control
long  cop_debug
long  cop_phase0
long  cop_monitor0
long  cop_monitor1
long  cop_config
long  cop_vram
long  cop_tile
long  cop_panx
long  cop_pany
long  cop_bgcolor
' |
' |
' LAST
long  cop_obj[obj_total_size]       ' 24 sprites


long bomb_state
long bomb_x
long bomb_y
long bomb_vel
long bomb_delay

long monster_state
long monster_x
long monster_y
long monster_vel_x

long boule_feu_state
long boule_feu_x
long boule_feu_y
long boule_feu_vel
long boule_feu_side
long boule_feu_index

long boule_feu_v_state
long boule_feu_v_x
long boule_feu_v_y
long boule_feu_v_vel
long boule_feu_v_index

long ballon_state
long ballon_vie
long ballon_x
long ballon_y
long ballon_index

long marker_x
long marker_y
long marker_delay

long lave_cnt
long lave_index
word lave_tile[4]

long time
long time_state

long time2
long time2_state

long pause_flag
long pause_state

long score

word madr

' -----------------------------------------------------------------------------

'///////////////////////////////////////////////////////////////////////
' OBJECT DECLARATION SECTION ///////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

OBJ

  manette  : "gamepad_drv_001.spin"
  cop      : "cop_drv_011.spin"                          
  tiles    : "fred_jeu_tiles.spin"                   
  map      : "fred_jeu_map.spin"                      
  
'///////////////////////////////////////////////////////////////////////
' PUBLIC FUNCTIONS /////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

PUB Start | debugled_dir, seed
'' This is the first entry point the system will see when the PChip starts,
'' execution ALWAYS starts on the first PUB in the source code for
'' the top level file

  ' start cop engine
  cop.setup(tiles.data,128,128, $00, map.data)          ' pass parameters to tile data and map data.
  cop.start(@cop_status)                                  ' start and boot two cogs   
  manette.start                           
  
  cop_bgcolor := $00000000                              ' background color quadpixel BLACK.
  cop_pany := 0                                         ' map pan position (0,0)
  cop_debug := $0                                       ' initial debug led intensity (0%)
  debugled_dir := $04000000                             ' increment by 1/64th of full intensity. / frame

  lave_cnt  := 0
  lave_index := 0
  monster_x := 0
  monster_y := 185
  monster_vel_x := 1
  monster_state := 20
  madr := map.data

  bomb_state := 0
  ballon_index := 30
  ballon_state := 1
  ballon_vie := 5
  ballon_x := SCREEN_WIDTH/2
  ballon_y := 10

  boule_feu_state := 0
  boule_feu_v_state := 0

  time_state := 0
  time2_state := 0
             
  pause_flag := 0 
  pause_state := 0

  score := 0
   
  repeat while TRUE
  
''  Pause
    if (manette.button(NES0_START) and (pause_flag == 0))
      pause_flag := 1
      if(pause_state == 0)
        pause_state := 1
      elseif(pause_state == 1)
        pause_state := 0
    if ((not manette.button(NES0_START)) and pause_flag == 1)
      pause_flag := 0
    if (pause_state == 1)
      next
        
''  Clear Sprites for new Frame.
    cop.newframe

''  Random temps horizontal
    if((boule_feu_state == 0) and (time_state == 0)) 
      time_state := 1
      time := cnt + clkfreq*((||(?seed)//2) + 1)

''  Random temps vertical
    if((boule_feu_v_state == 0) and (time2_state == 0)) 
      time2_state := 1
      time2 := cnt + clkfreq*((||(?seed)//12) + 4)

''  Crer boule horizontale
    if ((time_state == 1) and (cnt > time))
      time_state := 0
      boule_feu_state := 1

''  Crer boule verticale
    if ((time2_state == 1) and (cnt > time2))
      time2_state := 0
      boule_feu_v_state := 1

''  Rien encore    
    'if (manette.button(NES0_B) and (boule_feu_state == 0))
      'boule_feu_state := 1

''  Crer la bombe    
    if (manette.button(NES0_A) and (bomb_state == 0))
      bomb_x := ballon_x
      bomb_y := ballon_y
      bomb_vel := 3
      bomb_state := 1
      monster_state := 60

''  Deplacement ballon et gravit    
    if (manette.button(NES0_UP))
      ballon_y:= ballon_y-2
    if (manette.button(NES0_DOWN))
      ballon_y :=ballon_y+1 
    if (manette.button(NES0_LEFT))
      ballon_x:= ballon_x-1
    if (manette.button(NES0_RIGHT))
      ballon_x:= ballon_x+1

''  Collision ballon et ecran
    if (ballon_y< 0)
      ballon_y := 0
    if (ballon_y > 185)
      ballon_state := 0
    if (ballon_x< 0)
      ballon_x := SCREEN_WIDTH
    if (ballon_x> SCREEN_WIDTH)
      ballon_x := 0
       
    ballon_y++

    
'' Draw a Sprite
    'cop.sprite(10,100,20,20,0,0)


''  Random boule de feu ct et position initiale
    if (boule_feu_state == 1)
      boule_feu_side := (||(?seed))//2
      boule_feu_y := (||(?seed))//175 + 10
      boule_feu_state := 2
      if (boule_feu_side == 0)
        boule_feu_x := 0
        boule_feu_vel := 3
        boule_feu_index := 32 
      else
        boule_feu_x := SCREEN_WIDTH
        boule_feu_vel := -3
        boule_feu_index := 0

''  Collision boule de feu horizontale et ecran        
    if (boule_feu_state == 2)
      boule_feu_x += boule_feu_vel
      cop.sprite(boule_feu_x,boule_feu_y,32,16,boule_feu_index,80)
      if ((boule_feu_side == 0) and (boule_feu_x > SCREEN_WIDTH))
        boule_feu_state := 0
      if ((boule_feu_side == 1) and (boule_feu_x < 0))
        boule_feu_state := 0

''  Collision boule de feu horizontale et ballon
    if (boule_feu_state == 2)
      if ((boule_feu_x > ballon_x) and (boule_feu_x < (ballon_x+30)))
        if ((boule_feu_y > ballon_y) and (boule_feu_y < (ballon_y+32)))
          ballon_state := 0
          boule_feu_state := 0

''  Collision boule de feu verticale et ballon
    if (boule_feu_v_state == 3)
      if (((boule_feu_v_x+8) > ballon_x) and ((boule_feu_v_x+8) < (ballon_x+30)))
        if ((boule_feu_v_y > ballon_y) and (boule_feu_v_y < (ballon_y+32)))
          ballon_state := 0
          boule_feu_v_state := 0

''  Draw bombe, tuer monstre          
    if (bomb_state == 1)
      bomb_y += bomb_vel
      cop.sprite(bomb_x,bomb_y,16,16,16,64)
      if((bomb_x > monster_x) and (bomb_x < monster_x+32) and (bomb_y > 185))
        monster_state := 0
        bomb_state := 0
      if(bomb_y > SCREEN_HEIGHT)
        bomb_state := 0

    if (bomb_state == 3)
      if(bomb_delay-- < 0)
        bomb_state := 0
        
''  Couler monstre
    if (monster_state == 0)
      monster_y++
      bomb_state := 3
      if (monster_y > (SCREEN_HEIGHT + 20))
        monster_state := 20
        monster_y := 185
        bomb_state := 3
        bomb_delay := 200
        score++

''  Bouge le monstre
    ifnot(monster_state == 0)   
      monster_x += monster_vel_x
      if (monster_x < 0)
        monster_state := 40
        monster_vel_x *= -1      
      if (monster_x > 220)
        monster_state := 20
        monster_vel_x *= -1
     
    if (score > 15)
      ballon_index := 0
    
''  Draw monstre et ballon
    cop.sprite(monster_x,monster_y,32,20,96,monster_state)
    cop.sprite(ballon_x,ballon_y,30,32,36+ballon_index,0)

''  Cration boule de feu verticale et du X
    if (boule_feu_v_state == 1)
      boule_feu_v_x := ballon_x
      boule_feu_v_y := SCREEN_HEIGHT
      marker_x := ballon_x
      marker_y := ballon_y
      marker_delay := 20
      boule_feu_v_index := 0
      boule_feu_v_state := 2

    if (boule_feu_v_state == 2)
      cop.sprite(marker_x,marker_y,20,20,0,0)
      if(marker_delay-- < 0)
        boule_feu_v_state := 3

    if (boule_feu_v_state == 3)
      if(boule_feu_v_index == 0)
        cop.sprite(boule_feu_v_x,boule_feu_v_y,16,16,0,64)
        boule_feu_v_y -= 2
        if(boule_feu_v_y < marker_y)
          boule_feu_v_index := 1
      elseif(boule_feu_v_index == 1)
        cop.sprite(boule_feu_v_x,boule_feu_v_y,16,28,20,0)
        boule_feu_v_y += 2
        if(boule_feu_v_y > SCREEN_HEIGHT)
          boule_feu_v_state := 0

''  Calcul de la vie
    if (ballon_vie == 0)
      repeat '3000000
      ballon_vie := -1
    if (ballon_state == 0)
      ballon_x := SCREEN_WIDTH/2
      ballon_y := 10
      ballon_state := 1
      word[madr + ballon_vie*2 - 2] := $2030
      ballon_vie--

'' Adjust Debug LED
    cop_debug += debugled_dir                           
      if(cop_debug==0)
        debugled_dir := -debugled_dir
      else
        if(cop_debug==$fc000000)
          debugled_dir := -debugled_dir
        
'' Synchroniser vertical, animation lave et radom seed   
    LaveAnim
    seed := cnt  
    cop.waitvsync
   
PRI LaveAnim
  if (++lave_cnt => 20)
    lave_cnt := 0
    case lave_index
     0:
      lave_tile[0] := $3000
      lave_tile[1] := $3800
      lave_tile[2] := $3010
      lave_tile[3] := $3810 
     1:
      lave_tile[0] := $3020
      lave_tile[1] := $3820
      lave_tile[2] := $3030
      lave_tile[3] := $3830
     2:
      lave_tile[0] := $3040
      lave_tile[1] := $3840
      lave_tile[2] := $3050
      lave_tile[3] := $3850
     3:
      lave_tile[0] := $3060
      lave_tile[1] := $3860
      lave_tile[2] := $3070
      lave_tile[3] := $3870
    if (++lave_index => 4)
      lave_index := 0
       
    word[madr + 0 + 12<<5] := lave_tile[0]
    word[madr + 1 + 12<<5] := lave_tile[2]
    word[madr + 2 + 12<<5] := lave_tile[0]
    word[madr + 3 + 12<<5] := lave_tile[2]
    word[madr + 4 + 12<<5] := lave_tile[0]
    word[madr + 5 + 12<<5] := lave_tile[2]
    word[madr + 6 + 12<<5] := lave_tile[0]
    word[madr + 7 + 12<<5] := lave_tile[2]
    word[madr + 8 + 12<<5] := lave_tile[0]
    word[madr + 9 + 12<<5] := lave_tile[2]
    word[madr + 10 + 12<<5] := lave_tile[0]
    word[madr + 11 + 12<<5] := lave_tile[2]
    word[madr + 12 + 12<<5] := lave_tile[0]
    word[madr + 13 + 12<<5] := lave_tile[2]
    word[madr + 14 + 12<<5] := lave_tile[0]
    word[madr + 15 + 12<<5] := lave_tile[2]
    word[madr + 16 + 12<<5] := lave_tile[0]
    word[madr + 17 + 12<<5] := lave_tile[2]
    word[madr + 18 + 12<<5] := lave_tile[0]
    word[madr + 19 + 12<<5] := lave_tile[2]
    word[madr + 20 + 12<<5] := lave_tile[0]
    word[madr + 21 + 12<<5] := lave_tile[2]
    word[madr + 22 + 12<<5] := lave_tile[0]
    word[madr + 23 + 12<<5] := lave_tile[2]
    word[madr + 24 + 12<<5] := lave_tile[0]
    word[madr + 25 + 12<<5] := lave_tile[2]
    word[madr + 26 + 12<<5] := lave_tile[0]
    word[madr + 27 + 12<<5] := lave_tile[2]
    word[madr + 28 + 12<<5] := lave_tile[0]
    word[madr + 29 + 12<<5] := lave_tile[2]
    word[madr + 30 + 12<<5] := lave_tile[0]
    word[madr + 31 + 12<<5] := lave_tile[2]
     
    word[madr + 0 + 13<<5] := lave_tile[1]
    word[madr + 1 + 13<<5] := lave_tile[3]
    word[madr + 2 + 13<<5] := lave_tile[1]
    word[madr + 3 + 13<<5] := lave_tile[3]
    word[madr + 4 + 13<<5] := lave_tile[1]
    word[madr + 5 + 13<<5] := lave_tile[3]
    word[madr + 6 + 13<<5] := lave_tile[1]
    word[madr + 7 + 13<<5] := lave_tile[3]
    word[madr + 8 + 13<<5] := lave_tile[1]
    word[madr + 9 + 13<<5] := lave_tile[3]
    word[madr + 10 + 13<<5] := lave_tile[1]
    word[madr + 11 + 13<<5] := lave_tile[3]
    word[madr + 12 + 13<<5] := lave_tile[1]
    word[madr + 13 + 13<<5] := lave_tile[3]
    word[madr + 14 + 13<<5] := lave_tile[1]
    word[madr + 15 + 13<<5] := lave_tile[3]
    word[madr + 16 + 13<<5] := lave_tile[1]
    word[madr + 17 + 13<<5] := lave_tile[3]
    word[madr + 18 + 13<<5] := lave_tile[1]
    word[madr + 19 + 13<<5] := lave_tile[3]
    word[madr + 20 + 13<<5] := lave_tile[1]
    word[madr + 21 + 13<<5] := lave_tile[3]
    word[madr + 22 + 13<<5] := lave_tile[1]
    word[madr + 23 + 13<<5] := lave_tile[3]
    word[madr + 24 + 13<<5] := lave_tile[1]
    word[madr + 25 + 13<<5] := lave_tile[3]
    word[madr + 26 + 13<<5] := lave_tile[1]
    word[madr + 27 + 13<<5] := lave_tile[3]
    word[madr + 28 + 13<<5] := lave_tile[1]
    word[madr + 29 + 13<<5] := lave_tile[3]
    word[madr + 30 + 13<<5] := lave_tile[1]
    word[madr + 31 + 13<<5] := lave_tile[3]
     